package com.rshy.project.hy.aop;

import cn.hutool.core.util.StrUtil;
import com.rshy.project.hy.baseRe.BaseException;
import com.rshy.project.hy.baseRe.EnvEnum;
import com.rshy.project.hy.baseRe.ResultCode;
import com.rshy.project.hy.baseRe.Ret;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * @author dxy
 * @description: 异常处理类
 * @date 2021/4/5 16:48
 */
@RestControllerAdvice
@Slf4j
public class ExceptionAdvice {

    @Value("${spring.profiles.active}")
    private String env;

    /**
     * 参数校验失败时调用，返回json格式
     * @param e
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public Ret<?> validExceptionHandler(MethodArgumentNotValidException e) {
        log.error(e.getMessage(),e);
        FieldError fieldError = e.getBindingResult().getFieldError();
        assert fieldError != null;
        return Ret.error(ResultCode.VALIDATE_FAILED.getCode(),"[" + fieldError.getObjectName() + " " + fieldError.getField() + "]" + fieldError.getDefaultMessage());
    }

    @ExceptionHandler(MissingServletRequestParameterException.class)
    public Ret<?> bindExceptionHandler(MissingServletRequestParameterException e) {
        log.error(e.getMessage(),e);
        return Ret.error(ResultCode.VALIDATE_FAILED.getCode(),e.getMessage());
    }

    @ExceptionHandler(DuplicateKeyException.class)
    public Ret<?> bindExceptionHandler(DuplicateKeyException e) {
        log.error(e.getMessage(),e);
        return Ret.error(ResultCode.DUPLICATE.getCode(),e.getMessage());
    }


    /**
     * 通用星互动平台异常发生时调用，返回json格式
     * @param e
     * @return
     */
    @ExceptionHandler(value = BaseException.class)
    public Ret<?> handle(BaseException e) {
        log.error(e.getMessage(),e);
        if (e.getCode() != null) {
            return Ret.error(e.getCode(),e.getMessage());
        }
        return Ret.error(e.getCode(),e.getMessage());
    }

    /**
     * 通用异常发生时调用，返回json格式
     * @param e
     * @return
     */
    @ExceptionHandler
    public Ret<?> handle(Exception e) {
        log.error(e.getMessage(),e);
        String msg = e.getMessage();
        if (StrUtil.isBlank(msg)) {
            msg = e.toString();
        }
        return Ret.error(10000,"服务器打酱油啦~ 请稍后重试！");
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    public Ret<?> httperrMsgNotReadableExceptionHandler(HttpMessageNotReadableException e) {
        log.error("参数格式错误：" + e.getMessage(),e);
        if (env.equals(EnvEnum.PRODUCT.getCode())) {
            return Ret.error(10001,"参数错误");
        } else {
            return Ret.error(10001,e.getMessage());
        }
    }
}
