package com.rshy.project.hy.baseRe;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Getter;
import org.springframework.util.Assert;

import java.io.Serializable;

/**
 * @author Ye Jianyu
 * @date 2020/11/4
 */
@Getter
public final class Ret<T> implements Serializable {

    public static final Integer SUCCESSFUL_CODE = 1;
    public static final String SUCCESSFUL_errMsg = "成功";

    /**
     * 错误码
     *
     * @see ErrorCode#getCode()
     */
    private Integer errCode;

    /**
     * 错误提示，用户可阅读
     *
     * @see ErrorCode#geterrMsg()
     */
    private String errMsg;
    /**
     * 错误明细，内部调试错误
     */
    private String detailerrMsg;

    /**
     * 返回数据
     */
    @JsonInclude(JsonInclude.Include.NON_NULL)
    private T data;

    /**
     * 快速创建成功结果并返回结果数据
     *
     * @return Result
     */
    public static <T> Ret<T> success(T data) {
        Ret<T> result = new Ret<>();
        result.errCode = SUCCESSFUL_CODE;
        result.data = data;
        result.errMsg = SUCCESSFUL_errMsg;
        return result;
    }


    public static <T> Ret<T> error(ErrorCode errorCode) {
        return error(errorCode.getCode(), errorCode.geterrMsg(), null);
    }

    public static <T> Ret<T> error(Integer code, String errMsg) {
        return error(code, errMsg, null);
    }

    public static <T> Ret<T> error(Integer code, String errMsg, String detailerrMsg) {
        Assert.isTrue(!SUCCESSFUL_CODE.equals(code), "code 必须是错误的！");
        Ret<T> result = new Ret<>();
        result.errCode = code;
        result.errMsg = errMsg;
        result.detailerrMsg = detailerrMsg;
        return result;
    }

    public Ret<T> setDetailerrMsg(String detailerrMsg) {
        this.detailerrMsg = detailerrMsg;
        return this;
    }

    /**
     * 成功code=0
     *
     * @return true/false
     */
    @JsonIgnore
    public boolean isSuccess() {
        return SUCCESSFUL_CODE.equals(this.errCode);
    }

    /**
     * 失败
     *
     * @return true/false
     */
    @JsonIgnore
    public boolean isError() {
        return !isSuccess();
    }

    /**
     * 判断是否有异常。如果有，则抛出 {@link BaseException} 异常
     */
    public void checkError() throws ServiceException {
        if (isSuccess()) {
            return;
        }
        // 业务异常
        throw new ServiceException(errCode, errMsg).setDetailerrMsg(detailerrMsg);
    }

    public static <T> Ret<T> error(BaseException baseException) {
        return error(baseException.getCode(), baseException.getMessage(),
                baseException.getDetailerrMsg());
    }

    @Override
    public String toString() {
        return "CommonResult{" +
                "errCode=" + errCode +
                ", data=" + data +
                ", errMsg='" + errMsg + '\'' +
                ", detailerrMsg='" + detailerrMsg + '\'' +
                '}';
    }

}
